@@ -7,9 +7,11 @@ import (
 	. "github.com/onsi/gomega"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	operatorsv1 "github.com/operator-framework/api/pkg/operators/v1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/operators/decorators"
@@ -42,9 +44,52 @@ var _ = Describe("Operator Controller", func() {
 		Expect(k8sClient.Create(ctx, operator)).To(Succeed())
 	})
 
-	AfterEach(func() {
-		Expect(k8sClient.Get(ctx, name, operator)).To(Succeed())
-		Expect(k8sClient.Delete(ctx, operator, deleteOpts)).To(Succeed())
+	Describe("operator deletion", func() {
+		JustBeforeEach(func() {
+			Expect(k8sClient.Delete(ctx, operator)).To(Succeed())
+			Eventually(func() bool {
+				err := k8sClient.Get(ctx, name, operator)
+				return apierrors.IsNotFound(err)
+			}, timeout, interval).Should(BeTrue())
+		})
+		Context("with components bearing its label", func() {
+			var (
+				objs      []runtime.Object
+				namespace string
+			)
+
+			BeforeEach(func() {
+				namespace = genName("ns-")
+				objs = testobj.WithLabel(expectedKey, "",
+					testobj.WithName(namespace, &corev1.Namespace{}),
+				)
+
+				for _, obj := range objs {
+					Expect(k8sClient.Create(ctx, obj.(client.Object))).To(Succeed())
+				}
+			})
+
+			AfterEach(func() {
+				for _, obj := range objs {
+					Expect(k8sClient.Delete(ctx, obj.(client.Object), deleteOpts)).To(Succeed())
+				}
+				Expect(k8sClient.Delete(ctx, operator, deleteOpts)).To(Succeed())
+			})
+
+			It("should re-create it", func() {
+				Eventually(func() error {
+					return k8sClient.Get(ctx, name, operator)
+				}).Should(Succeed())
+			})
+		})
+		Context("with no components bearing its label", func() {
+			It("should not re-create it", func() {
+				Consistently(func() bool {
+					err := k8sClient.Get(ctx, name, operator)
+					return apierrors.IsNotFound(err)
+				}, timeout, interval).Should(BeTrue())
+			})
+		})
 	})
 
 	Describe("component selection", func() {
@@ -60,6 +105,10 @@ var _ = Describe("Operator Controller", func() {
 			}, timeout, interval).Should(Equal(expectedComponentLabelSelector))
 		})
 
+		AfterEach(func() {
+			Expect(k8sClient.Delete(ctx, operator, deleteOpts)).To(Succeed())
+		})
+
 		Context("with no components bearing its label", func() {
 			Specify("a status containing no component references", func() {
 				Consistently(func() ([]operatorsv1.RichReference, error) {
@@ -83,7 +132,7 @@ var _ = Describe("Operator Controller", func() {
 				)
 
 				for _, obj := range objs {
-					Expect(k8sClient.Create(ctx, obj)).To(Succeed())
+					Expect(k8sClient.Create(ctx, obj.(client.Object))).To(Succeed())
 				}
 
 				expectedRefs = toRefs(scheme, objs...)
@@ -91,8 +140,7 @@ var _ = Describe("Operator Controller", func() {
 
 			AfterEach(func() {
 				for _, obj := range objs {
-					Expect(k8sClient.Get(ctx, testobj.NamespacedName(obj), obj)).To(Succeed())
-					Expect(k8sClient.Delete(ctx, obj, deleteOpts)).To(Succeed())
+					Expect(k8sClient.Delete(ctx, obj.(client.Object), deleteOpts)).To(Succeed())
 				}
 			})
 
@@ -125,7 +173,7 @@ var _ = Describe("Operator Controller", func() {
 					)
 
 					for _, obj := range newObjs {
-						Expect(k8sClient.Create(ctx, obj)).To(Succeed())
+						Expect(k8sClient.Create(ctx, obj.(client.Object))).To(Succeed())
 					}
 
 					objs = append(objs, newObjs...)
@@ -143,7 +191,7 @@ var _ = Describe("Operator Controller", func() {
 			Context("when component labels are removed", func() {
 				BeforeEach(func() {
 					for _, obj := range testobj.StripLabel(expectedKey, objs...) {
-						Expect(k8sClient.Update(ctx, obj)).To(Succeed())
+						Expect(k8sClient.Update(ctx, obj.(client.Object))).To(Succeed())
 					}
 				})
 
